#	@(#) pbmconv.tcl 12.1 95/05/09 
#
#	Copyright (C) The Santa Cruz Operation, 1993-1994.
#	This Module contains Proprietary Information of
#	The Santa Cruz Operation, and should be treated as Confidential.
 
#	Author: Ronald Joe Record (rr@sco.com)
#	Date  : 03 Jun 1993
#	REVISION HISTORY:
#		05 Apr 1994 David Young (davidy@sco.com)
#			- ported to use new Widget Server syntax

# This script displays a list of graphical file formats and allows the
# user to indicate the file format s/he has and the format desired.
# The appropriate pbmplus pipe(s) are then constructed and the desired
# file created.

# Declare global scalar variables
global inpf outf inpExtlist inputIndex outExtlist outputIndex fileExt
global inputList outputList
# Declare global array variables
global Command

# List of supported input filename extensions
set inpExtlist "TIFF GIF PPM PGM PBM XBM IMG XPM PCX XWD"
# List of supported output filename extensions
set outExtlist "TIFF GIF PPM PGM PBM XBM IMG XPM PCX XWD"
# Initialize the input and output filenames
set inpf ""
set outf ""
#
# Set up the file format arrays.
#
set inputList "TIFF
GIF
PPM
PGM
PBM
{X10 or X11 bitmap file (.xbm)}
{GEM .img format}
{XPM (X Window System ASCII pixmaps)}
{PC Paintbrush .pcx format}
{X10 or X11 window dump file}"

set outputList "TIFF
GIF
PPM
PGM
PBM
{X10 or X11 bitmap file}
{GEM .img format}
{XPM (X Window System ASCII pixmaps)}
{PC Paintbrush .pcx format}
{X11 window dump file}"

set CommandList "
{true} {tifftopnm | ppmtogif} {tifftopnm} 
{tifftopnm | ppmtopgm}
{tifftopnm | ppmtopgm | pgmtopbm} {tifftopnm | pbmtoxbm} {tifftopnm | pbmtogem}
{tifftopnm | ppmtoxpm} {tifftopnm | ppmtopcx} {tifftopnm | pnmtoxwd} 

{giftoppm | pnmtotiff} {true} {giftoppm} {giftoppm | ppmtopgm}
{giftoppm | ppmtopgm | pgmtopbm} {giftoppm | ppmtopgm | pgmtopbm | pbmtoxbm}
{giftoppm | ppmtopgm | pgmtopbm | pbmtogem} {giftoppm | ppmtoxpm}
{giftoppm | ppmtopcx} {giftoppm | pnmtoxwd} 

{pnmtotiff} {ppmtogif} {true} {ppmtopgm} {ppmtopgm | pgmtopbm} 
{ppmtopgm | pgmtopbm | pbmtoxbm} {ppmtopgm | pgmtopbm | pbmtogem} 
{ppmtoxpm} {ppmtopcx} {pnmtoxwd}

{pnmtotiff} {pgmtoppm | ppmtogif} {pgmtoppm} {true} {pgmtopbm}
{pgmtopbm | pbmtoxbm} {pgmtopbm | pbmtogem} {pgmtoppm | ppmtoxpm}
{pgmtoppm | ppmtopcx} {pnmtoxwd}

{pnmtotiff} {false} {false} {false} {true} {pbmtoxbm} {pbmtogem} 
{false} {false} {pnmtoxwd}

{false} {false} {false} {false} {xbmtopbm} {true} 
{false} {false} {false} {false}

{false} {false} {false} {false} {gemtopbm} {false} {true} 
{false} {false} {false}

{xpmtoppm | pnmtotiff} {xpmtoppm | ppmtogif} {xpmtoppm} {xpmtoppm | ppmtopgm}
{xpmtoppm | ppmtopgm | pgmtopbm} {xpmtoppm | ppmtopgm | pgmtopbm | pbmtoxbm}
{xpmtoppm | ppmtopgm | pgmtopbm | pbmtogem} {true}
{xpmtoppm | ppmtopcx} {xpmtoppm | pnmtoxwd}

{pcxtoppm | pnmtotiff} {pcxtoppm | ppmtogif} {pcxtoppm} {pcxtoppm | ppmtopgm}
{pcxtoppm | ppmtopgm | pgmtopbm} {pcxtoppm | ppmtopgm | pgmtopbm | pbmtoxbm}
{pcxtoppm | ppmtopgm | pgmtopbm | pbmtogem} {pcxtoppm | ppmtoxpm}
{true} {xpmtoppm | pnmtoxwd}

{xwdtopnm | pnmtotiff} {xwdtopnm | ppmtogif} {xwdtopnm} {xwdtopnm | ppmtopgm}
{xwdtopnm | ppmtopgm | pgmtopbm} {xwdtopnm | ppmtopgm | pgmtopbm | pbmtoxbm}
{xwdtopnm | ppmtopgm | pgmtopbm | pbmtogem} {xwdtopnm | ppmtoxpm}
{xwdtopnm | ppmtopcx} {true} "

# Set up 2-D associative array with the first index representing the
# input file type, the second index representing the output file type
# and the value of an element representing the command or pipeline used
# for conversion from the input to output file types.
set i 0
set len [llength $CommandList]
foreach inpft $inputList {
	foreach outft $outputList {
		if { $i < $len } then {
			set Command($inpft,$outft) [lindex $CommandList $i]
		} else {
			set Command($inpft,$outft) true
		}
		incr i
	}
}

proc UsageMessage {} {
	echo "Usage: pbmplus \[-i inputfile\] \[-o outputfile\] \[-u\]"
	exit
}

proc Exitproc {cbs} {
	WsClose; exit
}

proc ScaleCB {cbs} {
	echo "This is supposed to handle scaling requests"
}

proc OptionsCB {cbs} {
	echo "This is supposed to handle desired options"
}

proc SetIndex {type} {
	global fileExt inpExtlist outExtlist
	if { $type == "input" } then {
		set extlist $inpExtlist
	} else {
		set extlist $outExtlist
	}
	foreach ext $extlist {
		if { $fileExt == $ext } then {
			return [lsearch $extlist $ext]
		}
	}
	return 0
}

proc GetFileExtType {file} {
	global fileExt
	set fileExt ""
	case [string tolower $file] in {
	   	{ *.pbm | *.pbm.* } { set fileExt PBM }
		{ *.pgm | *.pgm.* } { set fileExt PGM }
		{ *.ppm | *.ppm.* } { set fileExt PPM }
	   	{ *.x | *.x.* | *.xbm | *.xbm.* | *.x10bm | *.x10bm.* | *.x11bm | *.x11bm.* | *.bitmap | *.bitmap.* } { set fileExt XBM }
	   	{ *.r | *.r.* | *.rast | *.rast.* } { set fileExt RAS }
	   	{ *.mac | *.mac.* | *.macp | *.macp.* } { set fileExt MACP }
	   	{ *.g3 | *.g3.* | *.fax | *.fax.* } { set fileExt FAX }
	   	{ *.xwd | *.xwd.* | *.x10wd | *.x10wd.* | *.x11wd | *.x11wd.* } {
			set fileExt XWD }
	   	{ *.brush | *.brush.* } { set fileExt BRUSH }
	   	{ *.img | *.img.* } { set fileExt IMG }
	   	{ *.pcx | *.pcx.* } { set fileExt PCX }
	   	{ *.pic | *.pic.* | *.pict | *.pict.* | *.pict2 | *.pict2.* } { set fileExt PICT }
	   	{ *.tif | *.tif.* | *.tiff | *.tiff.* } { set fileExt TIFF }
	   	{ *.fs | *.fs.* | *.face | *.face.* } { set fileExt FS }
	   	{ *.hips | *.hips.* } { set fileExt HIPS }
	   	{ *.fits | *.fits.* } { set fileExt FITS }
	   	{ *.gif | *.gif.* } { set fileExt GIF }
	   	{ *.iff | *.iff.* | *.ilbm | *.ilbm.* } { set fileExt ILBM }
	   	{ *.lispm | *.lispm.* } { set fileExt LMB }
	   	{ *.mtv | *.mtv.* } { set fileExt MTV }
	   	{ *.qrt | *.qrt.* } { set fileExt QRT }
	   	{ *.tga | *.tga.* | *.targa | *.targa.* } { set fileExt TGA }
	   	{ *.xim | *.xim.* } { set fileExt XIM }
	   	{ *.xpm | *.xpm.* | *.xpm2 | *.xpm2.* } { set fileExt XPM }
	   	{ *.pi1 | *.pi1.* } { set fileExt PI1 }
	   	{ *.pi3 | *.pi3.* } { set fileExt PI3 }
	   	{ *.spu | *.spu.* } { set fileExt SPU }
	   	{ *.spc | *.spc.* } { set fileExt SPC }
	   	{ *.ybm | *.ybm.* | *.face | *.face.* } { set fileExt YBM }
	}
}

# First look for a magic number. If that fails, look at the filename extension
proc GetFileType {file} {
	global fileExt
	set fileExt ""
	# Location of magic file for the file binary (file -m $magic)
	set magic "/usr/local/lib/pbm/magic"

    case [exec file -m $magic $file] in {

	{ *PBM* } { set fileExt PBM }
	{ *PGM* } { set fileExt PGM }
	{ *PPM* } { set fileExt PPM }
	{ *compress* } {
		set newfile "/tmp/[id process]$file"
		exec rm -f $newfile
		exec zcat < $file > $newfile
		set file $newfile
		}
	{ *btoa* } {
		set newfile "/tmp/[id process]$file"
		exec rm -f $newfile
		exec atob < $file > "$newfile"
		set file $newfile
		}
	{ *Sun* } { set fileExt RAS }
	{ *rasterfile* } { set fileExt RAS }
	{ *GIF* } { set fileExt GIF }
	{ *TIFF* } { set fileExt TIFF }
	{ *IFF*ILBM* } { set fileExt ILBM }
	{ *Lisp* } { set fileExt LMB }
	{ *PC*Paintbrush* } { set fileExt PCX }
	{ *Bennet* } { set fileExt YBM }
	default {
		# Can't figure out the file type from the magic number,
		# so try the extension.
		GetFileExtType $file
		}
	}
}

proc inpf_okCB {cbs} {
	global inpf
	set inpf [keylget cbs selection]
	set dlog [keylget cbs dialog]
	WsSetValues [WxGetVar $dlog inputTextWidget]  \
		-value $inpf
	GetFileType $inpf
	set inputIndex [SetIndex "input"]
	WsSetValues [WxGetVar $dlog inputListListW] \
		-value [lindex $inputList $inputIndex]
}

proc outf_okCB {cbs} {
	global outf
	set outf [keylget cbs selection]
	set dlog [keylget cbs dialog]
	WsSetValues [WxGetVar $dlog outputTextWidget]  \
		-value $outf

	GetFileExtType $outf
	set outputIndex [SetIndex "output"]

	WsSetValues [WxGetVar $dlog outputListListW] \
		-value [lindex $outputList $outputIndex]

}

proc inpfCB {cbs} {
	set dlog [keylget cbs dialog]
	global inpf inputList inputIndex inpExtlist
	set fsb [WsFileSelectionDialog $dlog.input_fname \
		-title "Input file name" \
		-filter "[pwd]/*" \
		-okCallback inpf_okCB \
		]
	WsShow $fsb
}

proc outfCB {cbs} {
	global outf outputList outputIndex outExtlist

	set dlog [keylget cbs dialog]
	set fsb [WsFileSelectionDialog $dlog.fsb \
		-title "Output file name" \
		-filter "[pwd]/*" \
		-okCallback outf_okCB \
		]
	WsShow $fsb
}

proc Convert {cbs} {
	global inpf outf Command
	set dlog [keylget cbs dialog]
	set inputListListW [WxGetVar $dlog inputListListW]
	set outputListListW [WxGetVar $dlog outputListListW]

	set inpf_format [WsListGetSelectedItem $inputListListW -byItemList]
	set outf_format [WsListGetSelectedItem $outputListListW -byItemList]

	set command $Command($inpf_format,$outf_format)
	set prog [lindex $command 0]
	if { [system "$prog $inpf [lrange $command 1 end] > $outf"] } then {
		set dialog [WsErrorDialog $dlog.failure \
			-message "Conversion seems to have failed" \
			-cancelLabel Quit \
			-cancelCallback Exitproc \
			-okLabel Continue \
			]
	} else {
		set dialog [WsMessageDialog $dlog.success \
			-message "Conversion Completed Successfully" \
			-cancelLabel Quit \
			-cancelCallback Exitproc \
			-okLabel Continue \
			]
	}
	WsShow $dialog
}

proc OkCB {cbs} {
	global inpf outf
	if { $inpf == "" } then {
		inpfCB $cbs
	}
	if { $outf == "" } then {
		outfCB $cbs
	}
	Convert $cbs
}

#
# Process arguments, if any, and assign default selections based on extensions
#
set inputIndex 0
set outputIndex 0

while { [llength $argv] > 0 } {
	case [lindex $argv 0] in {
		-i	{ set inpf [lindex $argv 1] ; lvarpop argv 0 }
		-o	{ set outf [lindex $argv 1] ; lvarpop argv 0 }
		-u*	{ UsageMessage }
		default { lvarpop argv 0 }
	}
	lvarpop argv 0
}
GetFileType $inpf
set inputIndex [SetIndex "input"]
GetFileExtType $outf
set outputIndex [SetIndex "output"]

set app [WsOpen pbmfront]

set mainF [WsFormDialog $app.mainF \
	-title "Pbmplus Conversion Front-end" \
	-cancelLabel "Quit" \
	-cancelCallback Exitproc \
	-defaultButton OK \
	-okLabel "Make It So" \
	-okCallback OkCB \
	-wmDecoration ALL \
	]

#------
set labelstring "Select input and output filenames and formats"
set frame [WsFrame $mainF.frame \
	-shadowType ETCHED_OUT \
	]
set rowcol [WsRowColumn $frame.rowcol \
	]
set label1 [WsLabel $rowcol.desc \
	-label $labelstring \
	]
set sep [WsSeparator $mainF.sep \
	-topSide $label1 \
	-rightSide FORM \
	] 
#-----
set formLeft [WsForm $mainF.formLeft \
	-topSide $sep \
	]
set inputListListWLabel [WsLabel $formLeft.inputListListWLabel \
	-label "Input file format" \
	]
set inputListListW [WsList $formLeft.inputListListW \
	-topSide $inputListListWLabel \
	-itemList $inputList \
	-topOffset 0 \
	]
WsListSelectItem $inputListListW \
	-itemList [lindex $inputList $inputIndex] 

WxSetVar $mainF inputListListW $inputListListW

set formRight [WsForm $mainF.formRight \
	-leftSide $formLeft \
	-alignTop $formLeft \
	]

set outputListListWLabel [WsLabel $formRight.outputListListWLabel \
	-label "Output file format" \
	]
set outputListListW [WsList $formRight.outputListListW \
	-itemList $outputList \
	-topSide $outputListListWLabel \
	-topOffset 0 \
	]

WsListSelectItem $outputListListW \
	-itemList [lindex $outputList $outputIndex] 
WxSetVar $mainF outputListListW $outputListListW

set selectInFilePushB [WsPushButton $formLeft.selectInFilePushB \
	-label "Select Input File" \
	-callback inpfCB \
	-topSide $inputListListW \
	]

set selectOutFilePushB [WsPushButton $formRight.selectOutFilePushB \
	-label "Select Output File" \
	-callback outfCB \
	-topSide $outputListListW \
	]

set inputTextWidgetLabel [WsLabel $formLeft.inputTextWidgetLabel \
	-label "Input file name" \
	-topSide $selectInFilePushB \
	]
set inputTextWidget [WsText $formLeft.inputTextWidget \
	-value $inpf \
	-topSide $inputTextWidgetLabel \
	-topOffset 0 \
	-readOnly \
	]
WxSetVar $mainF inputTextWidget $inputTextWidget

set outputTextWidgetLabel [WsLabel $formRight.outputTextWidgetLabel \
	-label "Output file name" \
	-topSide $selectOutFilePushB \
	]
set outputTextWidget [WsText $formRight.outputTextWidget \
	-value $outf \
	-topSide $outputTextWidgetLabel \
	-topOffset 0 \
	-readOnly \
	]
WxSetVar $mainF outputTextWidget $outputTextWidget

set scalePushB [WsPushButton $formLeft.scalePushB \
	-label Scale \
	-callback ScaleCB \
	-topSide $inputTextWidget \
	]
set optionsPushB [WsPushButton $formRight.optionsPushB \
	-label Options \
	-callback OptionsCB \
	-topSide $outputTextWidget \
	]

WsShow $mainF
WsMainLoop
